﻿using Microsoft.Extensions.Logging;

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

using ProjectBoat_bae.DrawningObjects;
using ProjectBoat_bae.Generics;
using ProjectBoat_bae.Exceptions;
using ProjectBoat_bae.MovementStrategy;
using NLog;
using System.Numerics;

namespace ProjectBoat_bae
{
    public partial class FormBoatCollection : Form
    {
        // Набор объектов
        private readonly BoatsGenericStorage _storage;

        // Логер
        private readonly
            Microsoft.Extensions.Logging.ILogger? _logger;

        public FormBoatCollection(ILogger<FormBoatCollection> logger)
        {
            InitializeComponent();
            _storage = new BoatsGenericStorage(pictureBoxCollection.Width, pictureBoxCollection.Height);
            _logger = logger;
        }

        // Заполнение listBoxObjects
        private void ReloadObjects()
        {
            int index = listBoxStorages.SelectedIndex;
            listBoxStorages.Items.Clear();
            for (int i = 0; i < _storage.Keys.Count; i++)
            {
                listBoxStorages.Items.Add(_storage.Keys[i]);
            }
            if (listBoxStorages.Items.Count > 0 && (index == -1 || index
            >= listBoxStorages.Items.Count))
            {
                listBoxStorages.SelectedIndex = 0;
            }
            else if (listBoxStorages.Items.Count > 0 && index > -1 &&
            index < listBoxStorages.Items.Count)
            {
                listBoxStorages.SelectedIndex = index;
            }
        }

        // Добавление набора в коллекцию
        private void ButtonAddObject_Click_1(object sender, EventArgs e)
        {

            if (string.IsNullOrEmpty(textBoxStorageName.Text))
            {
                MessageBox.Show("Input not complete", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            _storage.AddSet(textBoxStorageName.Text.ToString());
            ReloadObjects();
            _logger.LogInformation($"Added set: {textBoxStorageName.Text}");
        }

        // Удаление набора
        private void ButtonDelObject_Click(object sender, EventArgs e)
        {
            
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            string name = listBoxStorages.SelectedItem.ToString() ?? string.Empty;
            if (MessageBox.Show($"Delete Object {name}?", "Deleting",
                MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                _storage.DelSet(name);
                ReloadObjects();
                _logger.LogInformation($"Deleted set: {name}");
            }
        }

        // Добавление объекта в набор
        private void ButtonAddBoat_Click(object sender, EventArgs e)
        {

            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            var obj = _storage[listBoxStorages.SelectedItem.ToString() ?? string.Empty];
            if (obj == null)
            {
                return;
            }
            FormBoatConfig formPlaneConfig = new FormBoatConfig();
            formPlaneConfig.AddEvent(AddBoat);
            formPlaneConfig.Show();
        }

        private void AddBoat(Drawningboat boat)
        {
            
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            var obj = _storage[listBoxStorages.SelectedItem.ToString() ?? string.Empty];
            if (obj == null)
            {
                _logger.LogWarning("Добавление пустого объекта");
                return;
            }
            try
            {
                _ = obj + boat;
                MessageBox.Show("Объект добавлен");
                pictureBoxCollection.Image = obj.ShowBoats();
                _logger.LogInformation($"plane added in set {listBoxStorages.SelectedItem.ToString()}");
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
                _logger.LogWarning($"plane not added in set {listBoxStorages.SelectedItem.ToString()}");
            }

        }

        //удаление объекта
        private void ButtonRemoveBoat_Click(object sender, EventArgs e)
        {

            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            var obj = _storage[listBoxStorages.SelectedItem.ToString() ?? string.Empty];
            if (obj == null)
            {
                return;
            }
            if (MessageBox.Show("Delete Object?", "Delete", MessageBoxButtons.YesNo,
                    MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }
            try
            {
                int pos = Convert.ToInt32(textBoxBoat.Text);
                if (obj - pos != null)
                {
                    MessageBox.Show("Object deleted");
                    pictureBoxCollection.Image = obj.ShowBoats();
                    _logger.LogInformation($"plane deleted in set {listBoxStorages.SelectedItem.ToString()}");
                }
                else
                {
                    MessageBox.Show("Object not deleted");
                    _logger.LogWarning($"plane not deleted in set {listBoxStorages.SelectedItem.ToString()}");
                }
            }
            catch (BoatNotFoundException ex)
            {
                MessageBox.Show(ex.Message);
                _logger.LogWarning($"PlaneNotFound: {ex.Message} in set {listBoxStorages.SelectedItem.ToString()}");
            }
            catch (Exception ex)
            {
                MessageBox.Show("Not input");
                _logger.LogWarning("Not input");
            }
        }

        // Обновление
        private void ButtonRefreshCollection_Click(object sender, EventArgs e)
        {

            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            var obj = _storage[listBoxStorages.SelectedItem.ToString() ?? string.Empty];
            if (obj == null)
            {
                return;
            }
            pictureBoxCollection.Image = obj.ShowBoats();
        }

        //Выбор набора
        private void listBoxStorages_SelectedIndexChanged(object sender, EventArgs e)
        {
            pictureBoxCollection.Image =
            _storage[listBoxStorages.SelectedItem?.ToString() ?? string.Empty]?.ShowBoats();
        }

        //сохранение
        private void SaveToolStripMenu_Click(object sender, EventArgs e)
        {

            if (saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    _storage.SaveData(saveFileDialog.FileName);
                    MessageBox.Show("Сохранение прошло успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    _logger.LogWarning($"Файл {saveFileDialog.FileName} успешно сохранен");
                }
                catch (Exception ex)
                {
                    _logger.LogWarning("Не удалось сохранить");
                    MessageBox.Show($"Не сохранилось: {ex.Message}", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);

                }
            }
        }

        //загрузка
        private void LoadToolStripMenu_Click(object sender, EventArgs args)
        {


            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    _storage.LoadData(openFileDialog.FileName);
                    _logger.LogInformation($"Данные загружены из файла {openFileDialog.FileName}");
                    ReloadObjects();
                }
                catch (Exception ex)
                {
                    MessageBox.Show($"Не загрузилось: {ex.Message}", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    _logger.LogWarning($"Не удалось загрузить информацию из файла: {ex.Message}");
                }
            }
        }

        private void ButtonSortByType_Click(object sender, EventArgs e) => CompareBoats(new BoatCompareByType());

        public void CompareBoats(IComparer<Drawningboat?> comparer)
        {
            if (listBoxStorages.SelectedIndex == -1) return;

            var obj = _storage[listBoxStorages.SelectedItem.ToString() ?? string.Empty];
            if (obj == null) return;

            obj.Sort(comparer);
            pictureBoxCollection.Image = obj.ShowBoats();
        }

        private void ButtonSortByColor_Click(object sender, EventArgs e) => CompareBoats(new BoatCompareByColor());

    }
}
