﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ProjectBoat_bae.MovementStrategy;
using ProjectBoat_bae.DrawningObjects;

namespace ProjectBoat_bae.Generics
{
    internal class BoatsGenericCollection <T, U>
        where T : Drawningboat
        where U : IMoveableObject
    {
        // Ширина окна прорисовки
        private readonly int _pictureWidth;

        // Высота окна прорисовки
        private readonly int _pictureHeight;
        /// <summary>
        /// Размер занимаемого объектом места (ширина)
        /// </summary>
        private readonly int _placeSizeWidth = 200;

        // Размер занимаемого объектом места (высота)
        private readonly int _placeSizeHeight = 200;

        // Набор объектов
        private readonly SetGeneric<T> _collection;
        
        // Конструктор
        public BoatsGenericCollection(int picWidth, int picHeight)
        {
            int width = picWidth / _placeSizeWidth;
            int height = picHeight / _placeSizeHeight;
            _pictureWidth = picWidth;
            _pictureHeight = picHeight;
            _collection = new SetGeneric<T>(width * height);
        }

        // Перегрузка оператора сложения
        public static bool operator +(BoatsGenericCollection<T, U> collect, T obj)
        {
            if (obj == null)
            {
                return false;
            }
            return collect._collection.Insert(obj, new DrawingBoatEqutables());
        }

        // Перегрузка оператора вычитания
        public static T? operator -(BoatsGenericCollection<T, U> collect, int
     pos)
        {
            T? obj = collect._collection[pos];
            if (obj != null)
            {
                collect._collection.Remove(pos);
            }
            return obj;
        }

        // Получение объекта IMoveableObject
        public U? GetU(int pos)
        {
            return (U?)_collection[pos]?.GetMoveableObject;
        }

        // Вывод всего набора объектов
        public Bitmap ShowBoats()
        {
            Bitmap bmp = new(_pictureWidth, _pictureHeight);
            Graphics gr = Graphics.FromImage(bmp);
            DrawBackground(gr);
            DrawObjects(gr);
            return bmp;
        }

        // Метод отрисовки фона
        private void DrawBackground(Graphics g)
        {
            Pen pen = new(Color.Black, 3);
            for (int i = 0; i < _pictureWidth / _placeSizeWidth; i++)
            {
                for (int j = 0; j < _pictureHeight / _placeSizeHeight +
                1; ++j)
                {//линия рамзетки места
                    g.DrawLine(pen, i * _placeSizeWidth, j *
                    _placeSizeHeight, i * _placeSizeWidth + _placeSizeWidth / 2, j *
                    _placeSizeHeight);
                }
                g.DrawLine(pen, i * _placeSizeWidth, 0, i *
                _placeSizeWidth, _pictureHeight / _placeSizeHeight * _placeSizeHeight);
            }
        }

        // Метод прорисовки объектов
        private void DrawObjects(Graphics g)
        {
            for (int i = 0; i < _collection.Count; i++)
            {
                T? t = _collection[i];
                if (t != null)
                {
                    t.SetPosition((i % (_pictureWidth / _placeSizeWidth)) * _placeSizeWidth, (i / (_pictureWidth / _placeSizeWidth)) * _placeSizeHeight);
                    if (t is Drawningboat) (t as Drawningboat).DrawTransport(g);
                    else t.DrawTransport(g);
                }
            }
        }

        // Получение объектов коллекции
        public IEnumerable<T?> GetBoats => _collection.GetBoats();
        public void Sort(IComparer<T?> comparer) => _collection.SortSet(comparer);
    }
}
