﻿using System;
using System.Collections.Generic;
using System.DirectoryServices.ActiveDirectory;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using ProjectBoat_bae.DrawningObjects;
using ProjectBoat_bae.MovementStrategy;
using ProjectBoat_bae.Entities;
using ProjectBoat_bae.Exceptions;

namespace ProjectBoat_bae.Generics
{
    internal class BoatsGenericStorage
    {
        readonly Dictionary<string, BoatsGenericCollection<Drawningboat, DrawningObjectBoat>> _boatStorages;

        public List<string> Keys => _boatStorages.Keys.ToList();

        private readonly int _pictureWidth;
        private readonly int _pictureHeight;
        public BoatsGenericStorage(int pictureWidth, int pictureHeight)
        {
            _boatStorages = new Dictionary<string, BoatsGenericCollection<Drawningboat, DrawningObjectBoat>>();
            _pictureWidth = pictureWidth;
            _pictureHeight = pictureHeight;
        }
        public void AddSet(string name)
        {
            
            if (_boatStorages.ContainsKey(name)) return;
            _boatStorages[name] = new BoatsGenericCollection<Drawningboat, DrawningObjectBoat>(_pictureWidth, _pictureHeight);
        }

        public void DelSet(string name)
        {
            if (!_boatStorages.ContainsKey(name)) return;
            _boatStorages.Remove(name);
        }

        public BoatsGenericCollection<Drawningboat, DrawningObjectBoat>?
            this[string ind]
        {
            get
            {
                if (_boatStorages.ContainsKey(ind)) return _boatStorages[ind];
                return null;
            }
        }

        private static readonly char _separatorForKeyValue = '|';
        private readonly char _separatorRecords = ';';
        private static readonly char _separatorForObject = ':';

        public void SaveData(string filename)
        {

            if (File.Exists(filename))
            {
                File.Delete(filename);
            }
            StringBuilder data = new();
            foreach (KeyValuePair<string,
            BoatsGenericCollection<Drawningboat, DrawningObjectBoat>> record in _boatStorages)
            {
                StringBuilder records = new();
                foreach (Drawningboat? elem in record.Value.GetBoats)
                {
                    records.Append($"{elem?.GetDataForSave(_separatorForObject)}{_separatorRecords}");
                }
                data.AppendLine($"{record.Key}{_separatorForKeyValue}{records}");
            }
            if (data.Length == 0)
            {
                throw new Exception("Невалидная операция, нет данных для сохранения");
            }
            using FileStream fs = new(filename, FileMode.Create);
            byte[] info = new
            UTF8Encoding(true).GetBytes($"BoatStorage{Environment.NewLine}{data}");
            fs.Write(info, 0, info.Length);
            return;

        }

        public void LoadData(string filename)
        {
            
            if (!File.Exists(filename))
            {
                throw new FileNotFoundException("Файл не найден");
            }

            using (StreamReader fs = File.OpenText(filename))
            {
                string str = fs.ReadLine();
                if (str == null || str.Length == 0)
                {
                    throw new ArgumentException("Нет объекта для добавления");
                }
                if (str.StartsWith("BoatsStorage"))
                {
                    throw new InvalidDataException("Неверный формат данных");
                }

                _boatStorages.Clear();
                string strs = "";

                while ((strs = fs.ReadLine()) != null)
                {
                    if (strs == null)
                    {
                        return;
                    }

                    string[] record = strs.Split(_separatorForKeyValue, StringSplitOptions.RemoveEmptyEntries);
                    if (record.Length != 2)
                    {
                        continue;
                    }
                    BoatsGenericCollection<Drawningboat, DrawningObjectBoat> collection = new(_pictureWidth, _pictureHeight);
                    string[] set = record[1].Split(_separatorRecords, StringSplitOptions.RemoveEmptyEntries);
                    foreach (string elem in set)
                    {
                        Drawningboat? boat = elem?.CreateDrawningBoat(_separatorForObject, _pictureWidth, _pictureHeight);
                        if (boat != null)
                        {
                            if (!(collection + boat))
                            {
                                try
                                {
                                    _ = collection + boat;
                                }
                                catch (BoatNotFoundException e)
                                {
                                    throw e;
                                }
                                catch (StorageOverflowException e)
                                {
                                    throw e;
                                }
                            }
                        }
                    }
                    _boatStorages.Add(record[0], collection);
                }
            }
        }
    }
}
