﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using ProjectBoat_bae.Exceptions;

namespace ProjectBoat_bae.Generics
{
    internal class SetGeneric<T> where T : class
    {
        private readonly List<T?> _places;
        public int Count => _places.Count;

        // Максимальное количество объектов в списке
        private readonly int _maxCount;

        public void SortSet(IComparer<T?> comparer) => _places.Sort(comparer);

        // Конструктор
        public SetGeneric(int count)
        {
            _maxCount = count;
            _places = new List<T?>(_maxCount);
        }

        public bool Insert(T boat, IEqualityComparer<T?>? equal = null)
        {
            if (equal != null)
            {
                foreach (var secondBoat in _places)
                {
                    if (equal.Equals(boat, secondBoat))
                    {
                        throw new Exception("Такой объект уже есть в коллекции");
                    }
                }
            }
            return Insert(boat, 0);
        }
        public bool Insert(T boat, int position, IEqualityComparer<T?>? equal = null)
        {
            if (position < 0 || position >= _maxCount)
                throw new StorageOverflowException("Impossible to insert");

            if (Count >= _maxCount)
                throw new StorageOverflowException(_maxCount);
            if (equal != null)
            {
                foreach (var secondBoat in _places)
                {
                    if (equal.Equals(boat, secondBoat))
                    {
                        throw new ApplicationException("Такой объект уже есть в коллекции");
                    }
                }
            }
            _places.Insert(position, boat);
            return true;
        }

        public bool Remove(int position)
        {
            if (position >= Count || position < 0)
                throw new BoatNotFoundException("Invalid operation");
            if (_places[position] == null)
                throw new BoatNotFoundException(position);
            _places.RemoveAt(position);
            return true;
        }
        public T? this[int position]
        {
            get
            {
                if (position < 0 || position > _maxCount)
                    return null;
                return _places[position];
            }
            set
            {
                if (position < 0 || position > _maxCount)
                    return;
                _places[position] = value;
            }
        }

        public IEnumerable<T?> GetBoats(int? maxBoats = null)
        {
            for (int i = 0; i < _places.Count; ++i)
            {
                yield return _places[i];
                if (maxBoats.HasValue && i == maxBoats.Value)
                {
                    yield break;
                }
            }
        }
    }

}